import pytest
import sys
from sqids import Sqids


def test_simple():
    sqids = Sqids()
    numbers = [1, 2, 3]
    id_str = "86Rf07"
    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


def test_different_inputs():
    sqids = Sqids()
    numbers = [0, 0, 0, 1, 2, 3, 100, 1_000, 100_000, 1_000_000, sys.maxsize]
    assert sqids.decode(sqids.encode(numbers)) == numbers


def test_incremental_numbers():
    sqids = Sqids()
    ids = {
        "bM": [0],
        "Uk": [1],
        "gb": [2],
        "Ef": [3],
        "Vq": [4],
        "uw": [5],
        "OI": [6],
        "AX": [7],
        "p6": [8],
        "nJ": [9],
    }
    for id_str, numbers in ids.items():
        assert sqids.encode(numbers) == id_str
        assert sqids.decode(id_str) == numbers


def test_incremental_numbers_same_index_0():
    sqids = Sqids()
    ids = {
        "SvIz": [0, 0],
        "n3qa": [0, 1],
        "tryF": [0, 2],
        "eg6q": [0, 3],
        "rSCF": [0, 4],
        "sR8x": [0, 5],
        "uY2M": [0, 6],
        "74dI": [0, 7],
        "30WX": [0, 8],
        "moxr": [0, 9],
    }
    for id_str, numbers in ids.items():
        assert sqids.encode(numbers) == id_str
        assert sqids.decode(id_str) == numbers


def test_incremental_numbers_same_index_1():
    sqids = Sqids()
    ids = {
        "SvIz": [0, 0],
        "nWqP": [1, 0],
        "tSyw": [2, 0],
        "eX68": [3, 0],
        "rxCY": [4, 0],
        "sV8a": [5, 0],
        "uf2K": [6, 0],
        "7Cdk": [7, 0],
        "3aWP": [8, 0],
        "m2xn": [9, 0],
    }
    for id_str, numbers in ids.items():
        assert sqids.encode(numbers) == id_str
        assert sqids.decode(id_str) == numbers


def test_multi_input():
    sqids = Sqids()
    numbers = list(range(100))
    output = sqids.decode(sqids.encode(numbers))
    assert numbers == output


def test_encoding_no_numbers():
    sqids = Sqids()
    assert sqids.encode([]) == ""


def test_decoding_empty_string():
    sqids = Sqids()
    assert sqids.decode("") == []


def test_decoding_invalid_character():
    sqids = Sqids()
    assert sqids.decode("*") == []


def test_encode_out_of_range_numbers():
    sqids = Sqids()
    with pytest.raises(ValueError):
        sqids.encode([-1])
    with pytest.raises(ValueError):
        sqids.encode([sys.maxsize + 1])



from sqids import Sqids, DEFAULT_ALPHABET

upper = 10_000


def test_uniques_with_padding():
    sqids = Sqids(min_length=len(DEFAULT_ALPHABET))
    unique_set = set()

    for i in range(upper):
        numbers = [i]
        id_ = sqids.encode(numbers)
        unique_set.add(id_)
        assert sqids.decode(id_) == numbers

    assert len(unique_set) == upper


def test_uniques_low_ranges():
    sqids = Sqids()
    unique_set = set()

    for i in range(upper):
        numbers = [i]
        id_ = sqids.encode(numbers)
        unique_set.add(id_)
        assert sqids.decode(id_) == numbers

    assert len(unique_set) == upper


def test_uniques_high_ranges():
    sqids = Sqids()
    unique_set = set()

    for i in range(100_000_000, 100_000_000 + upper):
        numbers = [i]
        id_ = sqids.encode(numbers)
        unique_set.add(id_)
        assert sqids.decode(id_) == numbers

    assert len(unique_set) == upper


def test_uniques_multi():
    sqids = Sqids()
    unique_set = set()

    for i in range(upper):
        numbers = [i, i, i, i, i]
        id_ = sqids.encode(numbers)
        unique_set.add(id_)
        assert sqids.decode(id_) == numbers

    assert len(unique_set) == upper



import pytest
import sys
from sqids import Sqids, DEFAULT_ALPHABET


def test_simple():
    sqids = Sqids(min_length=len(DEFAULT_ALPHABET))

    numbers = [1, 2, 3]
    id_str = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTM"

    assert sqids.encode(numbers) == id_str
    assert sqids.decode(id_str) == numbers


def test_incremental():
    numbers = [1, 2, 3]
    map = {
        6: "86Rf07",
        7: "86Rf07x",
        8: "86Rf07xd",
        9: "86Rf07xd4",
        10: "86Rf07xd4z",
        11: "86Rf07xd4zB",
        12: "86Rf07xd4zBm",
        13: "86Rf07xd4zBmi",
    }

    map[
        len(DEFAULT_ALPHABET) + 0
    ] = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTM"
    map[
        len(DEFAULT_ALPHABET) + 1
    ] = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMy"
    map[
        len(DEFAULT_ALPHABET) + 2
    ] = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMyf"
    map[
        len(DEFAULT_ALPHABET) + 3
    ] = "86Rf07xd4zBmiJXQG6otHEbew02c3PWsUOLZxADhCpKj7aVFv9I8RquYrNlSTMyf1"

    for min_length, id in map.items():
        sqids = Sqids(min_length=min_length)

        assert sqids.encode(numbers) == id
        assert len(sqids.encode(numbers)) == min_length
        assert sqids.decode(id) == numbers


def test_incremental_numbers():
    sqids = Sqids(min_length=len(DEFAULT_ALPHABET))

    ids = {
        "SvIzsqYMyQwI3GWgJAe17URxX8V924Co0DaTZLtFjHriEn5bPhcSkfmvOslpBu": [0, 0],
        "n3qafPOLKdfHpuNw3M61r95svbeJGk7aAEgYn4WlSjXURmF8IDqZBy0CT2VxQc": [0, 1],
        "tryFJbWcFMiYPg8sASm51uIV93GXTnvRzyfLleh06CpodJD42B7OraKtkQNxUZ": [0, 2],
        "eg6ql0A3XmvPoCzMlB6DraNGcWSIy5VR8iYup2Qk4tjZFKe1hbwfgHdUTsnLqE": [0, 3],
        "rSCFlp0rB2inEljaRdxKt7FkIbODSf8wYgTsZM1HL9JzN35cyoqueUvVWCm4hX": [0, 4],
        "sR8xjC8WQkOwo74PnglH1YFdTI0eaf56RGVSitzbjuZ3shNUXBrqLxEJyAmKv2": [0, 5],
        "uY2MYFqCLpgx5XQcjdtZK286AwWV7IBGEfuS9yTmbJvkzoUPeYRHr4iDs3naN0": [0, 6],
        "74dID7X28VLQhBlnGmjZrec5wTA1fqpWtK4YkaoEIM9SRNiC3gUJH0OFvsPDdy": [0, 7],
        "30WXpesPhgKiEI5RHTY7xbB1GnytJvXOl2p0AcUjdF6waZDo9Qk8VLzMuWrqCS": [0, 8],
        "moxr3HqLAK0GsTND6jowfZz3SUx7cQ8aC54Pl1RbIvFXmEJuBMYVeW9yrdOtin": [0, 9],
    }

    for id_str, numbers in ids.items():
        assert sqids.encode(numbers) == id_str
        assert sqids.decode(id_str) == numbers


def test_min_lengths():
    for min_length in [0, 1, 5, 10, len(DEFAULT_ALPHABET)]:
        for numbers in [
            [0],
            [0, 0, 0, 0, 0],
            [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
            [100, 200, 300],
            [1000, 2000, 3000],
            [1000000],
            [sys.maxsize],
        ]:
            sqids = Sqids(min_length=min_length)

            id_str = sqids.encode(numbers)
            assert len(id_str) >= min_length
            assert sqids.decode(id_str) == numbers


def test_out_of_range_invalid_min_length():
    with pytest.raises(ValueError):
        Sqids(min_length=-1)

    with pytest.raises(ValueError):
        Sqids(min_length=256)
